// This is a roxen module. (c) On The Verge 2001
//
// The zope relay module. 
// his module provides a zope front end for handling virtual servers,
// ssl and other stuff
//
//  This code is (c) 2000 On The Verge.
//  It can be used, modified and redistributed freely 
//  under the terms of the GNU General Public License, version 2.
//
//  This code comes on a AS-IS basis, with NO WARRANTY OF ANY KIND, either
//  implicit or explicit. Use at your own risk.
//  You can modify this code as you wish, but in this case please
//  - state that you changed the code in the modified version
//  - do not remove our name from it
//  - send us a copy of the modified version or a patch, so that
//    we can include it in the 'official' release.
//  If you find this code useful, please e-mail us. It would definitely
//  boost our ego :)
//  
//  For risks and side-effects please read the code or ask your local 
//  unix or roxen-guru.

#include <module.h>
constant cvs_version = "$Id: zope.pike,v 1.6 2001/03/15 21:28:28 mbaehr Exp $";

#include <module.h>
inherit "module";
inherit "roxenlib";

#ifndef TYPE_URL
// need this until caudium gets TYPE_URL
#define TYPE_URL TYPE_STRING
#endif

void create()
{
  defvar("zope", "", "Zope Server URL", TYPE_URL, 
         "The Zope Server you want to access (only http is supported)"
        );

  defvar("mountpoint", "/", "Mountpoint", TYPE_STRING,
         "Where Zope will be mounted in the site's virtual file system.<br>"
         "Usually you want this to be /", 0, lambda(){return query("virtual");}
        ); 

  defvar("virtual", 1, "Handle Virtual Hosting", TYPE_FLAG,
         "Turn this on if you want to handle Virtual Hosts in Zope"
        );

  defvar("virtualhosts", "default /", "Virtual Hosts", TYPE_TEXT_FIELD,
         "Syntax:<pre>"
         "    ab.domain.com       /ab/\n"
         "    bc.domain.com       /bc/\n"
         "    main.domain.com     /\n"
         "    default             /serverlist/"
         "</pre>", 0, lambda(){return !query("virtual");}
        );
}

array register_module()
{
  return ({ MODULE_LOCATION, "Zope Gateway",
            doc()
            ,0,0 });
}

string query_location()
{
  return query("mountpoint");
}

string doc()
{
  return "this module provides a zope front end for handling virtual servers,"
         " ssl and other stuff";
}

mapping virtualhosts = ([]);

void start()
{
  if(query("virtual"))
  {
    array a;
    string s;
    virtualhosts = ([]);
    foreach(replace(query("virtualhosts"), "\t", " ")/"\n", s)
    {
      a = s/" " - ({""});
      if(sizeof(a)>=2) {
        virtualhosts[lower_case(a[0])] = a[1];
      }
    }
  }
}

mixed find_file( string f, object id )
{
  array zopeurl=array_sscanf(query("zope"), "http://%s:%d/");
  string zopehost=zopeurl[0];
  int zopeport=zopeurl[1]||80;

  mapping headers=copy_value(id->request_headers);
  m_delete(headers, "keep-alive");
  headers->connection="close";

  string path="/";

  if(query("virtual"))
    if(!(path=(virtualhosts[id->host]||virtualhosts->default)))
      return 0;

  string request_data = (id->method+" "
                      + http_encode_string(path+f+(id->query?"?"+id->query:""))
                      + " HTTP/1.1\r\n"
                      + encode_headers( headers ) + "\r\n" 
                      + id->data );

  Stdio.File fd = Stdio.File( );
  fd->async_connect(zopehost, zopeport, connected, id, request_data, fd);

#ifdef CAUDIUM
  id->do_not_disconnect = 1; 
  // net to set this to stop caudium from closing the connection 
#endif

  return http_pipe_in_progress( );
}

// this function is taken and modified from Roxen WebServer 2.1:

void connected(int how, object id, string request_data, Stdio.File fd)
{
  if( !how )
  {
    id->send_result( ([
      "type":"text/plain",
      "data":"Connection to remote HTTP host failed."
    ]) );
    fd->close();
  }
  else
  {
    // Send headers to remote server. (non-blocking)
    Stdio.sendfile(({ request_data }), id->my_fd, 0, -1, 0, fd);
    Stdio.sendfile(0, fd, 0, -1, 0, id->my_fd);
  }
}

// this function is taken from Roxen WebServer 2.1:

string encode_headers( mapping q )
{
  string res = "";
  string content_length="";
  foreach( sort( indices( q ) ), string h )
    if(lower_case(h)=="content-length")
      content_length = (string)h+": "+(string)q[h]+"\r\n";
    else
      res += (string)h+": "+(string)q[h]+"\r\n";
  return res+content_length;
}

